# NB: F-stat is an approximation here since the RSS1 was
# derived from full model with feature variable set to zero
# and compared with RSS2 from the full model with original input

#### Input:
# X: N x T mRNA expression matrix of N genes measured in T samples;
# Z: M x T miRNA expression matrix of M miRNA measured in T samples;
# cnv, met: N x T copy number variation or methylation scores of N genes
# measured in T samples;
# mirtar: N x M mRNA-miRNA target interaction matrix;
# tfbs: N x K TF-gene regulatory matrix for K TFs.


#### Output:
# feat.stats: Explanatory power of each regulators for mRNA expression
# in terms of F-statistics (see manuscript for more details)

feature_selection <- function(X, Z, cnv, met, mirtar, tfbs) {
	
	set.seed(rand.seed) # random seed for the 10-fold cv
	
	# step 1: estimate TF and miRNA activities per sample
	model.list <- mclapply(1:ncol(X), function(i) {
	
		X.t <- X[,i]
		cnv.t <- cnv[,i]
		met.t <- met[,i]	
		Z.t <- Z[,i]
		
		get_model(X.t, cnv.t, met.t, mirtar, Z.t, tfbs)
	})
	
	candidate.feat <- c(colnames(tfbs), colnames(mirtar))
	
	total.feat <- length(candidate.feat) + 2
	
	feat.stats <- mclapply(candidate.feat, function(feat) {
		
		rss1 <- 0
		
		rss2 <- 0
		
		for(j in 1:ncol(X)) {
			
			mylist <- model.list[[j]]
			
			inp1 <- mylist$inp
			
			inp1[,feat] <- 0
			
			pred1 <- predict(mylist$fit, inp1, s="lambda.min")
			
			pred2 <- predict(mylist$fit, mylist$inp, s="lambda.min")
			
			rss1 <- rss1 + sum((pred1 - mylist$resp)^2)
			
			rss2 <- rss2 + sum((pred2 - mylist$resp)^2)
		}
		
		rss1 <- rss1/ncol(X)
		
		rss2 <- rss2/ncol(X)
		
		df2 <- nrow(X) - total.feat + 1
		
		fstats <- (rss1 - rss2)/(rss2/df2)
		
		data.frame(feat=feat, fstats=fstats, 
			pval=pf(fstats, 1, df2, lower.tail=FALSE))
	})
		
	feat.stats <- do.call("rbind", feat.stats)
	
	feat.stats$FDR <- p.adjust(feat.stats$pval, "fdr")
	
	feat.stats
}


# Estimate TF and miRNA activities per sample
get_model <- function(X.t, cnv.t, met.t, mirtar, Z.t, tfbs) {
	
	# N
	resp <- as.numeric(X.t)
	
	mir.obs <- mirtar * matrix(rep(Z.t, nrow(mirtar)), nrow(mirtar), byrow=TRUE)
	
	# N x D
	inp <- cbind(cnv.t, met.t, mir.obs, tfbs)
	
	colnames(inp) <- c("CNV", "Met", colnames(mir.obs), colnames(tfbs))
	
	list(fit=cv.glmnet(inp, resp), inp=inp, resp=resp)
}








