library(glmnet)
library(parallel)

#### Input:
# X: N x T mRNA expression matrix of N genes measured in T samples;
# Z: M x T miRNA expression matrix of M miRNA measured in T samples;
# cnv, met: N x T copy number variation or methylation scores of N genes
# measured in T samples;
# mirtar: N x M mRNA-miRNA target interaction matrix;
# tfbs: N x K TF-gene regulatory matrix for K TFs.


#### Output:
# a list containing:
# W_g.mir: N x M predicted interactions between N genes and M miRNAs;
# W_g.tf: N x K predicted interactions between N genes and K TFs;
# alpha: a list containing the inferred activities of CNV, Met, miRNA and TF.

racer <- function(X, Z, cnv, met, mirtar, tfbs, rand.seed=100) {
	
	set.seed(rand.seed) # random seed for the 10-fold cv
	
	# step 1: estimate TF and miRNA activities per sample
	alpha <- mclapply(1:ncol(X), function(i) {
	
		X.t <- X[,i]
		cnv.t <- cnv[,i]
		met.t <- met[,i]	
		Z.t <- Z[,i]
		
		get_activities(X.t, cnv.t, met.t, mirtar, Z.t, tfbs)
	})
	
	alpha <- do.call("rbind", alpha)
	
	# step 2: estimate TF-g and miR-g associations across sample
	
	# use miRNA activities as predictor
	W <- mclapply(1:nrow(X), function(i)
		get_interactions(X[i,], alpha, 
			cnv[i,], met[i,], tfbs[i,], mirtar[i,]))
	
	# use original miRNA expression and seed as predictor
	# W <- mclapply(1:nrow(X), function(i)
		# get_interactions(X[i,], alpha, cnv[i,], met[i,], tfbs[i,], mirtar[i,], Z))

	W <- do.call("rbind", W)
	
	mir.a <- apply(alpha[,colnames(mirtar)],2,mean)
	
	W_g.mir <- W[,colnames(W) %in% colnames(mirtar)] *
			matrix(rep(mir.a, nrow(W)), nrow(W), byrow=T)
		
	W_g.tf <- W[,colnames(W) %in% colnames(tfbs)]
	
	
	list(W_g.mir = W_g.mir, W_g.tf = W_g.tf, alpha = alpha)
}


# Estimate TF and miRNA activities per sample
get_activities <- function(X.t, cnv.t, met.t, mirtar, Z.t, tfbs) {
	
	# N
	resp <- as.numeric(X.t)
	
	mir.obs <- mirtar * matrix(rep(Z.t, nrow(mirtar)), nrow(mirtar), byrow=TRUE)
	
	# N x D
	inp <- cbind(cnv.t, met.t, mir.obs, tfbs)
	
	colnames(inp) <- c("CNV", "Met", colnames(mir.obs), colnames(tfbs))
	
	alpha <- coef(cv.glmnet(inp, resp), s="lambda.min")[-1]
	
	names(alpha) <- colnames(inp)
	
	alpha
}


# Estimate TF-g and miR-g interactions across sample 
get_interactions <- function(X.i, act, cnv.i, met.i, tfbs.i, mirtar.i) {
	
	W.i <- matrix(rep(0, ncol(act)), nrow=1)
	
	colnames(W.i) <- colnames(act)
				
	reg <- c(names(which(mirtar.i > 0)), names(which(tfbs.i > 0)))
	
	inp <- cbind(cnv.i, met.i, act[,reg])
	
	W.i[,c("CNV","Met",reg)] <- coef(cv.glmnet(inp, X.i), s="lambda.min")[-1]	
	
	W.i
}



